// Copyright (c) 2016, Herman Bergwerf. All rights reserved.
// Use of this source code is governed by an AGPL-3.0-style license
// that can be found in the LICENSE file.

// THIS FILE IS AUTO-GENERATED, PLEASE DO NOT MODIFY BY HAND.

part of molview_messages.v0;

/// Searching messages
/// ==================
/// These messages should be used for extensive search operations such as
/// full-text searches, graph searches, or even SQL queries. There are also
/// messages to provide search autocomplete data for databases.

/// Request autocomplete data for the given search string.
class FindSearchSuggestions extends OmnibusRequest {
  /// omnibusId
  static const id = 'molview_messages.v0.FindSearchSuggestions';

  /// Input text
  final String text;

  /// Number of desired results per category
  final int count;

  /// Constructor
  FindSearchSuggestions(this.text, [this.count = 10]) : super(id);

  /// Construct from passive data structure.
  factory FindSearchSuggestions.fromPds(final Map<String, dynamic> pds) {
    String text;
    int count;

    // Read text.
    if (pds.containsKey('text') && pds['text'] is String) {
      text = pds['text'] as String;
    } else {
      throw new ArgumentError.value(
          pds, 'pds', "does not contain a valid field for property 'text'");
    }

    // Read count.
    if (pds.containsKey('count') && pds['count'] is int) {
      count = pds['count'] as int;
    } else {
      throw new ArgumentError.value(
          pds, 'pds', "does not contain a valid field for property 'count'");
    }

    return new FindSearchSuggestions(text, count);
  }

  /// Generate passive data structure from message properties.
  Map<String, dynamic> toPds() {
    return {'text': text, 'count': count};
  }
}

/// Message handler for FindSearchSuggestions
class FindSearchSuggestionsHandler extends OmnibusRequestHandler {
  /// Callback
  final OmnibusRequestCallback<FindSearchSuggestions> _callback;

  /// Constructor
  FindSearchSuggestionsHandler(this._callback, StateData state)
      : super(FindSearchSuggestions.id, state);

  /// Process incoming message.
  Future<List<Future<OmnibusResponse>>> process(OmnibusRequest data) async {
    FindSearchSuggestions message = data is FindSearchSuggestions
        ? data
        : new FindSearchSuggestions.fromPds(data.toPds());

    return [_callback(message, state)];
  }
}

/// Return a list of autocomplete suggestions.
class SearchSuggestions extends OmnibusResponse {
  /// omnibusId
  static const id = 'molview_messages.v0.SearchSuggestions';

  /// Origin database name
  final String database;

  /// Primarily related table name
  final String table;

  /// Key data class of suggestions (should be in the given table)
  final String key;

  /// List of autocomplete suggestions
  final List<SearchSuggestion> suggestions;

  /// Constructor
  SearchSuggestions(this.database, this.table, this.key, this.suggestions)
      : super(id);

  /// Construct from passive data structure.
  factory SearchSuggestions.fromPds(final Map<String, dynamic> pds) {
    String database;
    String table;
    String key;
    List<SearchSuggestion> suggestions;

    // Read database.
    if (pds.containsKey('database') && pds['database'] is String) {
      database = pds['database'] as String;
    } else {
      throw new ArgumentError.value(
          pds, 'pds', "does not contain a valid field for property 'database'");
    }

    // Read table.
    if (pds.containsKey('table') && pds['table'] is String) {
      table = pds['table'] as String;
    } else {
      throw new ArgumentError.value(
          pds, 'pds', "does not contain a valid field for property 'table'");
    }

    // Read key.
    if (pds.containsKey('key') && pds['key'] is String) {
      key = pds['key'] as String;
    } else {
      throw new ArgumentError.value(
          pds, 'pds', "does not contain a valid field for property 'key'");
    }

    // Read suggestions.
    if (pds.containsKey('suggestions') && pds['suggestions'] is List) {
      suggestions = pdsToList(
          pds['suggestions'],
          (Map<String, dynamic> item) => new SearchSuggestion.fromPds(
              new Map<String, dynamic>.from(item))) as List<SearchSuggestion>;
    } else {
      throw new ArgumentError.value(pds, 'pds',
          "does not contain a valid field for property 'suggestions'");
    }

    return new SearchSuggestions(database, table, key, suggestions);
  }

  /// Generate passive data structure from message properties.
  Map<String, dynamic> toPds() {
    return {
      'database': database,
      'table': table,
      'key': key,
      'suggestions': pdsFromList(suggestions)
    };
  }
}

/// Create SearchSuggestions Stream from the given responses.
Stream<SearchSuggestions> streamSearchSuggestions(OmnibusResponses responses) {
  return responses.stream(
          SearchSuggestions.id,
          (OmnibusResponse data) => data is SearchSuggestions
              ? data
              : new SearchSuggestions.fromPds(data.toPds()))
      as Stream<SearchSuggestions>;
}
